/* Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; version 3.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1
import Ubuntu.Components.Popups 0.1

Item {
    id: addStocksRect
    anchors.fill: parent
    property alias searchTextAlias: addStockTextField.text
    property bool matchIndicator: true
    property string active_search_tick: ""
    property string before_after_space: "00"
    //   This is used as two varibles > 0 is for has space at begining of line in `function update_search_text`

    Timer {
        id: reloadTrigger
        interval: 250; running: false; repeat: false
        onTriggered: {
            mainViewAdd.enabled = false
            currentStocks.reload()
            mainViewAdd.enabled = true
        }
     }
    ListModel {
          id: searchMatches
          objectName: "searchMatches"
    }


    function update_search_text(selected_word, old_word, before_after) {
        log("update_search_text")
        var before_space = '', after_space = ''
        if ( before_after[0] > 0 ) before_space = ' '
        if ( before_after[1] > 0 ) after_space = ' '
        // before_space and after_space are needed for the first and last word regex matching
        addStockTextField.text = addStockTextField.text.replace(before_space + old_word + after_space, before_space + selected_word.toLowerCase() + after_space)
        addStockTextField.text = addStockTextField.text.replace(/ +/g, " ")+" "
    }


    function text_change_handler() {
        log("text_change_handler()")

        // Find the current word the user is editing by spaces before and after current word at cursor
        var cursor_to_end = addStockTextField.text.substr(addStockTextField.cursorPosition).split(' ')
        var cursor_to_begining = addStockTextField.text.substr(0, addStockTextField.cursorPosition+cursor_to_end[0].length).split(' ')
        var changed_text = cursor_to_begining[cursor_to_begining.length-1]
        if ( !searchbox.open_box && changed_text.length > 0 ) {
             PopupUtils.open(autoCompletePopup, addStockTextField)
             searchbox.open_box = true
        }
        if ( changed_text.length > 0 && searchbox.open_box )
             load_matches(changed_text, (cursor_to_begining.length-1).toString()[0]+""+(cursor_to_end.length-1).toString()[0])
    }

    function load_matches(search_word, before_after) {
        var xhr = new XMLHttpRequest();
        matchIndicator = true
        active_search_tick = search_word
        before_after_space = before_after
        xhr.open("GET","http://d.yimg.com/autoc.finance.yahoo.com/autoc?query="+search_word+"&callback=YAHOO.Finance.SymbolSuggest.ssCallback", true);
        xhr.onreadystatechange = function()
        {
            if ( xhr.readyState == xhr.DONE) {
                if ( xhr.status == 200 ) {
                    var YAHOO = {
                        Finance: {
                            SymbolSuggest: {
                                ssCallback: function (data) {
                                    if ( !data.ResultSet.Result.length ) {
                                        searchMatches.clear()
                                        if ( search_word === 'steckroth')
                                            searchMatches.set(0, {"symbol": 'R.E.S.', "name": "Robert Steckroth", "exchDisp": "Bust0ut"})
                                        else
                                            searchMatches.set(0, {"symbol": 'No matches found for "'+data.ResultSet.Query+'"', "name": "", "exchDisp": ""})
                                    }
                                    else {
                                        for ( var a = 0; a < data.ResultSet.Result.length; a++ )
                                            searchMatches.set(a, {"symbol": data.ResultSet.Result[a].symbol,
                                                                  "name": data.ResultSet.Result[a].name, "exchDisp": data.ResultSet.Result[a].exchDisp})
                                    }
                                }
                            }
                        }
                    }
                    matchIndicator = false
                }
                eval( '(' + xhr.responseText + ')' )
            }
        }
        xhr.send()
    }

    Component {
        id: autoCompletePopup
        Popover {

            objectName: "autoCompletePopup"
            autoClose: true
            Column {
                anchors {
                    top: parent.top
                    left: parent.left
                    right: parent.right
                }
                height: addStocksRect.height/2
                Header {
                    id: popoverHeader
                    text: i18n.tr("Matching: "+active_search_tick)
                    ActivityIndicator {
                        objectName: "autocompleteActivity"
                        anchors.centerIn: parent
                        height: units.gu(2)
                        width: height
                        clip: true
                        running: matchIndicator
                    }
                }
                ListView {
                    clip: true
                    objectName: "matchListView"
                    width: parent.width
                    height: parent.height
                    model: searchMatches
                    delegate:
                        Standard {
                            text: i18n.tr(symbol+" "+name+" "+exchDisp)
                            onClicked: {
                                update_search_text(symbol, active_search_tick, before_after_space)
                                hide()
                                addStockTextField.focus = false
                                searchbox.open_box = false
                                if ( active_search_tick === 'steckroth')
                                    developerPopup.show()
                                reload_stocks(currentStocksList, addStockTextField.text)
                            }
                        }
                }
            }
            onVisibleChanged: {
                if ( !visible ) {
                    searchbox.open_box = false
                    addStockTextField.text = ""
                }
            }
        }
    }

    Item {
        id: searchbox
        objectName: "searchbox"
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height:  units.gu(3);
        property TextField input: addStockTextField
        property bool open_box: false
        TextField {
            id : addStockTextField
            objectName: "addStockTextField"
            placeholderText: i18n.tr("Search for Stocks (space separated)")
            anchors.fill: parent
            anchors.leftMargin: units.gu(1)
            anchors.rightMargin: units.gu(1)
            onTextChanged: text_change_handler()
        }
        Rectangle {
           radius: 2
           smooth: true
           anchors.left: parent.left
           anchors.right: parent.right
           anchors.top: parent.bottom
           height: units.gu(3)-4
           gradient: Gradient {
               GradientStop { position: 0; color: "#2b3847" }
               GradientStop { position: 0.5; color: "#354C68" }
               GradientStop { position: 1; color: "#2b3847" }
           }
           Text {
               color: "white"
               text: i18n.tr("powered by Yahoo Finance ®")
               font.pixelSize: FontUtils.sizeToPixels("small")
               anchors.centerIn: parent
           }

           Item {
               objectName: "managementAdd"
               anchors.top: parent.bottom
               anchors.left: parent.left
               anchors.right: parent.right
               height: addStocksRect.height - parent.height - searchbox.height
               ListView {
                  id: mainViewAdd
                  objectName: "mainViewAdd"
                  property bool oneShot: true
                  property real newAngle: 0.0
                  anchors.fill: parent
                  clip: true
                  onContentYChanged: {
                      if ( visibleArea.yPosition === 0 ) {
                          oneShot = true
                      }
                      else if ( -visibleArea.yPosition*1800 > 360 && oneShot ) {
                          oneShot = false
                          reloadTrigger.start()
                      }
                      else if ( -visibleArea.yPosition*1800 > 360 ) {
                          newAngle = -360
                      }
                      else if ( -visibleArea.yPosition*1800 > 180 ){
                          newAngle = -180
                      }
                      else {
                          newAngle = 0
                      }
                  }
                  model: currentStocksList
                  delegate: Stock_Component_Add { }
                  header:
                     Item {
                         width: parent.width
                         Image {
                             id: pullDownImage
                             objectName: "pullDownImage"
                             anchors.bottom: parent.bottom
                             anchors.horizontalCenter: parent.horizontalCenter
                             source:  "img/stock_refresh.png"
                             opacity: -mainViewAdd.contentY > 2 ? 1 : 0;
                             Behavior on opacity { NumberAnimation { duration: 200  } }
                             rotation: mainViewAdd.newAngle
                             Behavior on rotation { NumberAnimation { duration: 150 } }
                          }
                     }
                }
           }
      }
    }

    ActivityIndicator {
        objectName: "addActivityIndicator"
         anchors.centerIn: parent
         clip: false
         running: currentStocks.status != XmlListModel.Ready
   }

    Dialog {
        id: developerPopup
        title: i18n.tr("Stock Ticker")
        text: i18n.tr("from Surgemcgee.")
        Button {
            text: i18n.tr("Ok")
            onClicked: PopupUtils.close(developerPopup)
        }
    }


}










