/* Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; version 3.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import QtQuick.XmlListModel 2.0

Item {
    property alias mainViewCurrent: mainViewCurrent // used for positionViewAtIndex calls a lot
    property int day_range: 30
    Timer {
        id: reloadTrigger
        interval: 200; running: false; repeat: false
        onTriggered: {
            mainViewCurrent.enabled = false  // This will disable the mouse press in the listview and auto-return it
            currentStocks.reload()
            mainViewCurrent.enabled = true
        }
    }
    Rectangle { // Used to replace empty ListModels with a pretty logo/page
        radius: 6
        visible: currentStocksList.count > 0 ? false : true
        anchors.fill: parent
        anchors.margins: units.gu(1)
        color: visible ? "#969ca6" : "lightgrey"
        Image {
            visible: parent.height-50 > 32 && parent.width-50 > 32 // Turn off if the logo goes below 26 pixels
            anchors.centerIn: parent
            source: "svg/stock_ticker_logo.svg"
            sourceSize.height: visible && parent.height-50 // Stop the SVG rasterization if the object is not visible
            sourceSize.width: visible && parent.width-50  // Stop the SVG rasterization if the object is not visible
        }
    }
    Rectangle {
         anchors.fill: parent
         objectName: "testRectThin"
         color: "#e5e5e5"
         clip: true
         visible: currentStocksList.count ? true : false
         ListView {
             id: mainViewCurrent
             objectName: "mainViewCurrent"
             anchors.fill: parent
             property bool oneShot: true
             property real newAngle: 0.0
             onContentYChanged: {
                 if ( visibleArea.yPosition === 0 ) {
                     oneShot = true
                 }
                 else if ( -visibleArea.yPosition*1800 > 360 && oneShot ) {
                     oneShot = false
                     reloadTrigger.start()
                 }
                 else if ( -visibleArea.yPosition*1800 > 360 ) {
                     newAngle = -360
                 }
                 else if ( -visibleArea.yPosition*1800 > 180 ) {
                     newAngle = -180
                 }
                 else {
                     newAngle = 0
                 }
             }

             onCurrentIndexChanged: {
                 if ( count > 0 ) { detailsPage.detailViewCurrent.positionViewAtIndex(currentIndex, ListView.Center)
                                    current_index = currentIndex; chartTick = currentStocksList.get(currentIndex).tick }
             }
             model: currentStocksList
             snapMode:  ListView.SnapPosition
             delegate: Stock_Component_Main {}
             onCountChanged: {
                 if ( count <= 0 ) chartTick = "" // We do not want "undefined" when this count is zero
                 else if ( count === 1 ) current_index = currentIndex; // This is used when the first stock is added. The index needs to update from a blank list
             }
             header:  Item {
                            width: parent.width
                            Image {
                                anchors.bottom: parent.bottom
                                anchors.horizontalCenter: parent.horizontalCenter
                                source:  "img/stock_refresh.png"
                                opacity: -mainViewCurrent.contentY > 2 ? 1 : 0
                                Behavior on opacity { NumberAnimation { duration: 250 } }
                                rotation: mainViewCurrent.newAngle
                                Behavior on rotation { NumberAnimation { duration: 150 } }

                           }
                      }
         }
         ActivityIndicator {
             objectName: "currentActivityIndicator"
             anchors.centerIn: parent
             clip: false
             running: currentStocks.status != XmlListModel.Ready
         }
         Scrollbar {
             flickableItem: mainViewCurrent
             align: Qt.AlignTrailing
         }
     }
}
