/* Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; version 3.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem

Rectangle {
    id: stockComponentRect
    property string detailLableColor: "#010B14"
    property string detailValueColor: "#0E0E0E"
    width: parent.width
    radius: 6
    color: "#FCFCFC"
    height: childrenRect.height
    Column {
        width: parent.width
        property int borderSize: 20
        Item {
            width: childrenRect.width
            height: childrenRect.height
            anchors.horizontalCenter: parent.horizontalCenter
            Row {
                spacing: 12
                Text {
                    text: i18n.tr(cmpy_name)
                    font.bold: true
                    font.pixelSize: FontUtils.sizeToPixels("large")
                    color: current_index == index ? "#BD732F" : "#1A0F04"
                }
                Text {
                    text: i18n.tr('('+tick+')')
                    font.bold: false
                    font.pixelSize: FontUtils.sizeToPixels("meduim")
                    color: "#1A0F04"//current_index == index ? "#BD732F" : "#1A0F04"
                }
            }
        }
        ListItem.ThinDivider {}
        Item {
            width: parent.width -  parent.borderSize
            height: childrenRect.height
            anchors.horizontalCenter: parent.horizontalCenter
            Text {
               anchors.left: parent.left
               text: i18n.tr("Close"+(open !== "N/A" ? "/Open:" : ":") )
               font.bold: true
               font.pixelSize: FontUtils.sizeToPixels("medium")
               color: detailLableColor
            }
            Text {
                anchors.right: parent.right
                text: i18n.tr(close+(open !== "N/A" ? "/"+open : ""))
                font.bold: true
                font.pixelSize: FontUtils.sizeToPixels("medium")
                color: detailValueColor
            }
        }
        Item {
            visible: ask !== "N/A" && buy !== "N/A"
            width: parent.width - parent.borderSize
            height: childrenRect.height
            anchors.horizontalCenter: parent.horizontalCenter
            Text {
                anchors.left: parent.left
                text: i18n.tr("Ask/Buy")
                font.bold: true
                font.pixelSize: FontUtils.sizeToPixels("medium")
                color: detailLableColor
            }
            Text {
                anchors.right: parent.right
                text: i18n.tr(ask+"/"+buy)
                font.bold: true
                font.pixelSize: FontUtils.sizeToPixels("medium")
                color: detailValueColor
            }
        }
        Item {
             width: parent.width - parent.borderSize
             height: childrenRect.height
             anchors.horizontalCenter: parent.horizontalCenter
             Text {
                 anchors.left: parent.left
                 text: i18n.tr("52 Week low/high:")
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailLableColor
             }
             Text {
                 anchors.right: parent.right
                 text: i18n.tr(low_52_week+"/"+high_52_week)
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailValueColor
             }
         }
         Item {
             visible: low_day !== "N/A"
             width: parent.width - parent.borderSize
             height: childrenRect.height
             anchors.horizontalCenter: parent.horizontalCenter
             Text {
                 anchors.left: parent.left
                 text: i18n.tr("Todays low/high:")
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailLableColor
             }
             Text {
                 anchors.right: parent.right
                 text: i18n.tr(low_day+"/"+high_day)
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailValueColor
             }
         }
         Item {
             visible: dividend !== "N/A"
             width: parent.width - parent.borderSize
             height: childrenRect.height
             anchors.horizontalCenter: parent.horizontalCenter
             Text {
                 anchors.left: parent.left
                 text: i18n.tr("Dividend Per Share:")
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailLableColor
             }
             Text {
                 anchors.right: parent.right
                 text: i18n.tr(dividend)
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailValueColor
             }
         }
         Item {
             visible: volume !== "N/A"
             width: parent.width - parent.borderSize
             height: childrenRect.height
             anchors.horizontalCenter: parent.horizontalCenter
             Text {
                 anchors.left: parent.left
                 text: i18n.tr("Volume:")
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailLableColor
             }
             Text {
                 anchors.right: parent.right
                 text: i18n.tr(addCommas(volume))
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailValueColor
             }
         }
         Item {
              visible: after_hours !== "N/A"
              width: parent.width - parent.borderSize
              height: childrenRect.height
              anchors.horizontalCenter: parent.horizontalCenter
              Text {
                  anchors.left: parent.left
                  text: i18n.tr("After hours:")
                  font.bold: true
                  font.pixelSize: FontUtils.sizeToPixels("medium")
                  color: detailLableColor
              }
              Text {
                  anchors.right: parent.right
                  text: i18n.tr(after_hours)
                  font.bold: true
                  font.pixelSize: FontUtils.sizeToPixels("medium")
                  color: detailValueColor
              }
          }
         // P/E Ration:
         Item {
             visible: pe_ratio !== "N/A"
             width: parent.width - parent.borderSize
             height: childrenRect.height
             anchors.horizontalCenter: parent.horizontalCenter
             Text {
                 anchors.left: parent.left
                 text: i18n.tr("P/E Ratio:")
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailLableColor
             }
             Text {
                 anchors.right: parent.right
                 text: i18n.tr(pe_ratio)
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailValueColor
             }
         }
         Item {
             visible: eps !== "N/A - N/A"
             width: parent.width - parent.borderSize
             height: childrenRect.height
             anchors.horizontalCenter: parent.horizontalCenter
             Text {
                 anchors.left: parent.left
                 text: i18n.tr("EPS (current year):")
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailLableColor
             }
             Text {
                 anchors.right: parent.right
                 text: i18n.tr(eps)
                 font.bold: true
                 font.pixelSize: FontUtils.sizeToPixels("medium")
                 color: detailValueColor
             }
         }
         Item {
              visible: market_cap !== "N/A - N/A"
              width: parent.width - parent.borderSize
              height: childrenRect.height
              anchors.horizontalCenter: parent.horizontalCenter
              Text {
                  anchors.left: parent.left
                  text: i18n.tr("Market Cap")
                  font.bold: true
                  font.pixelSize: FontUtils.sizeToPixels("medium")
                  color: detailLableColor
              }
              Text {
                  anchors.right: parent.right
                  text: i18n.tr(market_cap)
                  font.bold: true
                  font.pixelSize: FontUtils.sizeToPixels("medium")
                  color: detailValueColor
              }
         }
    }
}
