/* Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; version 3.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem

import QtQuick.XmlListModel 2.0
import QtQuick.LocalStorage 2.0
import "javascript/storage.js" as Storage

MainView {
    // objectName for functional testing purposes (autopilot-qt5)
    objectName: "stockTicker"
    applicationName: "stock-ticker-app"
    width: units.gu(40)
    height: units.gu(75)
    id: mainProgram

    property string chartTick: ""
    property int chart_day_range: 30
    property string acsdes: "up"  // Sorts the model in ascending or descending
    property real refa: 2.9 // The height of the subtabs pages
    property int current_index: -1
    property int sub_tab_index: 1
    property string changedFormat: "%"
    property int chart_index: 1
    property string lastSortType: "Name"
    property string errorText: ""
    property int dummyTextWidth: 0  // Holds the width of the percent sign in Stock_Component_Main

    property string serverDomain: "http://www.surgemcgee.com:8080/"
 //   property string serverDomain: "http://localhost:8080/"

    property string quoteParams: "?f=sabl1vpghojkc8dre7j1n&s="

    function log(message) {
         console.log("["+applicationName+"] "+message)
    }

    Component.onCompleted: {
        Storage.initialize()
        sub_tab_index = Storage.getSetting("sub_tab_index") || sub_tab_index
        current_index = Storage.getSetting("last_current_index") || current_index
        rootTabs.selectedTabIndex = Storage.getSetting("last_page") || rootTabs.selectedTabIndex
        refa = Storage.getSetting("refa_pos") || refa
        acsdes = Storage.getSetting("acsdes_order") || acsdes
        acsdes === "up" ? acsdes = "down" : acsdes = "up"  // We want to start sorting the opposite direction from which was saved
        chart_day_range = parseInt( Storage.getSetting("chart_day_range") || chart_day_range )

        currentStocks.source = Storage.getSetting("current_stocks_source") || currentStocks.source+'yhoo'
        lastSortType = Storage.getSetting("last_sort_type") || lastSortType

        // Get out graph settings
        chartsPage.changeSettings({showDayRange: Storage.getSetting("graph_show_day_range")})
        if ( Storage.getSetting("changed_format") === changedFormat )
            changedFormat === "%" ? changedFormat = "$" : changedFormat = "%"

        log("MainView loaded")
    }

    Component.onDestruction: {
        Storage.setSetting("changed_format", changedFormat)
        Storage.setSetting("last_page", rootTabs.selectedTabIndex)
        Storage.setSetting("chart_day_range", chart_day_range)
        Storage.setSetting("last_current_index", currentPage.mainViewCurrent.currentIndex)
        Storage.setSetting("acsdes_order", acsdes)
        Storage.setSetting("last_sort_type", lastSortType)
        Storage.setSetting("refa_pos", refa)
        Storage.setSetting("sub_tab_index", sub_tab_index)
        Storage.setSetting("graph_show_day_range", chartsPage.getSettings("showDayRange"))
    }

    function setChangedFormat() {
        changedFormat === "%" ? changedFormat = "$" : changedFormat = "%"
        for ( var i = 0; i < currentStocksList.count; i++ )
            currentStocksList.setProperty(i, "priceFormat", changedFormat)
    }

    function getPrice(price, close, open) {
        if ( ((price+2)%(price+3)) )
             return price
        else if ( ((open+2)%(open+3)) )
                   return open
        else if ( ((close+2)%(close+3)) )
                  return close
        else return "---"
    }

    function getChangedAmount(close, price, format) {
        if ( !((close+2)%(close+3)) )
             close = price
        var changed_amount_int = (price-close).toFixed(2)
        if ( format === "%") changed_amount_int = (100/(close/changed_amount_int)).toFixed(2)
        if ( changed_amount_int < 0 && format !== 'goNegative' ) changed_amount_int *= -1
        return changed_amount_int
    }


    function reload_stocks(listModel, params) {
        log("reload_stocks()")
        params = params || ""
        var new_source = serverDomain+quoteParams

        var current_stock_tick = []
        for ( var x = 0; x < listModel.count; x++ )
            current_stock_tick.push(listModel.get(x).tick.toLowerCase())

        current_stock_tick = current_stock_tick.concat(params.toLowerCase().split(' '))
        // Remove duplicates from params
        var obj = {}
        for ( var u = 0; u < current_stock_tick.length; u++ )
              obj[current_stock_tick[u]] = 0
        for ( var p in obj )
              new_source += "+"+p

        if ( new_source.toString() === decodeURIComponent(listModel.source).toString() )
            currentStocks.reload()
        else
            currentStocks.source = new_source
    }

    function addCommas(num_char) {
        var comma_num =  ""
        var f = 1;
        for ( var x = num_char.length-1; x >= 0; x-- ) {
              comma_num = num_char.charAt(x) + ""+comma_num
              if ( !(f % 3) && x > 0 ) comma_num =  ","+comma_num
              f++
        }
        return comma_num
    }

    function sortModel(listModel, type, order) {
        var modelA, modelB
        if ( order )
            acsdes = order
        if ( type )
            lastSortType = type
        else
            type = lastSortType

        for ( var n = 0; n < listModel.count; n++)
           for ( var i = n+1; i < listModel.count; i++) {
                 // Reverse A and B for reverse order
               if ( type === "Volume" ) {
                   modelA = parseInt(listModel.get(n).volume)
                   modelB = parseInt(listModel.get(i).volume)
               }
               else if ( type === "Changed" ) {
                   modelA = parseFloat(getChangedAmount(listModel.get(n).close, getPrice(listModel.get(n).price, listModel.get(n).close, listModel.get(n).open), changedFormat))
                   modelB = parseFloat(getChangedAmount(listModel.get(i).close, getPrice(listModel.get(i).price, listModel.get(i).close, listModel.get(i).open), changedFormat))
               }
               else if ( type === "Price" ) {
                   modelA = parseFloat(getPrice(listModel.get(n).price, listModel.get(n).close, listModel.get(n).open))
                   modelB = parseFloat(getPrice(listModel.get(i).price, listModel.get(i).close, listModel.get(i).open))
               }
               else if ( type === "Name" ) {
                   modelA = listModel.get(n).cmpy_name
                   modelB = listModel.get(i).cmpy_name
               }
               if ( order === "down" && modelA > modelB ) {
                   listModel.move(i, n, 1);
                   n=0;
               }
               else if ( order === "up" && modelB > modelA ) {
                   listModel.move(i, n, 1);
                   n=0;
               }
           }
        currentPage.mainViewCurrent.positionViewAtIndex(currentPage.mainViewCurrent.currentIndex, ListView.Contain)
        // Save new order to the XMLListModel source for persistants in sorting order with exit-start
        var n_tick = serverDomain+quoteParams
        for ( var f = 0; f < currentStocksList.count; f++ )
            n_tick += "+"+currentStocksList.get(f).tick.toLowerCase()
        Storage.setSetting("current_stocks_source", n_tick)
    }

    function manageModelList(baseModel, listModel, currentStock, append) {
        var startIndex = 0, priceFormat = "%", baseCount = baseModel.count, listCount = listModel.count
        if ( typeof ( append ) === "boolean" && append )
             startIndex = baseCount

        for ( var i = 0; i < listCount; i++ ) {
            var modListEle = listModel.get(i)
            if ( typeof ( modListEle ) !== "undefined" && typeof ( modListEle.priceFormat ) !== "undefined" && modListEle.priceFormat.length > 0 )
                priceFormat = modListEle.priceFormat
            else if ( typeof ( baseModel.get(i) ) !== "undefined" && typeof ( baseModel.get(i).priceFormat ) !== "undefined" && baseModel.get(i).priceFormat.length > 0 )
                priceFormat = baseModel.get(i).priceFormat

            baseModel.set(startIndex+i, {"cmpy_name": modListEle.cmpy_name, "price": parseFloatElse(modListEle.price), "close": parseFloatElse(modListEle.close),
                                           "tick": modListEle.tick, "volume": modListEle.volume, "open": parseFloatElse(modListEle.open), "priceFormat": priceFormat, "isCurrent": currentStock,
                                           "low_52_week": parseFloatElse(modListEle.low_52_week), "high_52_week": parseFloatElse(modListEle.high_52_week), "low_day": parseFloatElse(modListEle.low_day),
                                           "high_day": parseFloatElse(modListEle.high_day), "dividend": parseFloatElse(modListEle.dividend), "after_hours": parseFloatElse(modListEle.after_hours),
                                           "ask": parseFloatElse(modListEle.ask), "buy": parseFloatElse(modListEle.buy), "pe_ratio": parseFloatElse(modListEle.pe_ratio),
                                           "eps": modListEle.eps, "market_cap": modListEle.market_cap})
        }
        for ( var x = listCount+startIndex; x < baseCount; x++ ) {
              baseModel.remove(x)
              x--; baseCount--
        }
    }

    function parseFloatElse(thing) {
        return parseFloat(thing) && parseFloat(thing).toFixed(2) || "N/A"
    }

    function control_tab(sub_tab_new, skip_checking) {
        if ( sub_tab_new === sub_tab_index && !skip_checking ) return

        animateOpacityPageOff.target = currentRectSizer.children[sub_tab_index]
        animateOpacityPageOn.target = currentRectSizer.children[sub_tab_new]

        animateColorTabOff.target = subtabsPage.tab_id.children[sub_tab_index]
        animateColorTabOff.start()

        currentRectSizer.children[sub_tab_index].visible = false
        animateColorTabOn.target = subtabsPage.tab_id.children[sub_tab_new]
        animateColorTabOn.start()

        sub_tab_index = sub_tab_new

        currentRectSizer.children[sub_tab_new].visible = true

        animateOpacityPageOff.start()
        animateOpacityPageOn.start()
    }

    PropertyAnimation { id: animateColorTabOff; property: "gradientColor"; to: "#12384D"; duration: 15; }
    PropertyAnimation { id: animateColorTabOn; property: "gradientColor"; to: "#1EACAE"; duration: 25; }
    NumberAnimation { id: animateOpacityPageOff; property: "opacity"; from: 1.0; to: 0.3; duration: animateColorTabOff.duration }
    NumberAnimation { id: animateOpacityPageOn; property: "opacity"; from: 0.3; to: 1.0; duration: animateColorTabOn.duration }

    function reset_application() {
        control_tab(1, true) // or make all subtabs visable=false
        chartTick = ""
        chart_day_range = 30
        acsdes = "up"  // Sorts the model in ascending or descending
        refa = 2.9 // The height of the subtabs pages
        changedFormat = "%"
        current_index = -1
        sub_tab_index = 1
        currentStocksList.clear()
        newsFeedList.clear()
        currentStocks.source = serverDomain+quoteParams+'yhoo'
        rootTabs.selectedTabIndex = 0
        chartsPage.changeSettings('defaults')
        lastSortType = "Name"
    }

    Component {
        id: resetApp
        Dialog {
            id: resetDialogue
            objectName: "resetDataDialog"
            title: "Reset Application Data"
            text: "Warning! This will clear all data from the entire application to the defaults."
            Button {
                objectName: "cancelButton"
                text: "Cancel"
                onClicked: PopupUtils.close(resetDialogue)
            }
            Button {
                objectName: "resetConfirm"
                text: "Erase all history and data"
                color: "orange"
                onClicked: { PopupUtils.close(resetDialogue); reset_application(); }
            }
        }
    }
    Component {
        id: chartSheetConfig
            DefaultSheet {
                id: chartSheet
                title: i18n.tr("Chart Settings")
                doneButton: true
                onDoneClicked: PopupUtils.close(chartSheet)
                Column {
                    width: Math.round(parent.width * 0.85)
                    spacing: units.gu(1)
                    Row {
                        spacing: units.gu(2)
                        Text {
                            anchors.verticalCenter: parent.verticalCenter
                            text: i18n.tr("Show graph day ranges")
                        }
                        Switch {
                            checked: chartsPage.getSettings("showDayRange") === "yes"
                            onCheckedChanged: chartsPage.changeSettings({showDayRange: checked ? "yes" : "no"})
                        }
                    }
                }
            }
        }
    Dialog {
        id: errorPopupDialog
        title: i18n.tr("Connection Error")
        text: errorText
        Button {
            text: i18n.tr("Try Again")
            onClicked: {
                PopupUtils.close(errorPopupDialog)
                currentStocks.reload()
            }
        }
        Button {
            text: i18n.tr("Quit Application")
            color: "orange"
            onClicked: Qt.quit()
        }
    }


    Timer {
        id: connection_error
        interval: 10000; running: false; repeat: false
        onTriggered: errorPopupDialog.show()
    }

    ListModel {
          id: currentStocksList
    }

    ListModel {
        id: newsFeedList
    }

    XmlListModel {
        id: newsFeed
        source: "http://finance.yahoo.com/rss/headline?s="+chartTick
        query: "/rss/channel/item"
        XmlRole { name: "title"; query: "title/string()" }
        XmlRole { name: "description"; query: "description/string()" }
        XmlRole { name: "link"; query: "link/string()" }

        onStatusChanged: {
            if ( status === XmlListModel.Loading )
                log("Loading newsFeed: "+source)
            if ( status === XmlListModel.Error ) {
                errorText = errorString()+"\n"+i18n.tr("Check your connection and/or firewall settings.")
                connection_error.start()
                log("Error loading "+source+" : "+errorString())
            }
            if ( status === XmlListModel.Ready ) {
                connection_error.stop()
                for ( var y = 0; y < count; y++ )
                    newsFeedList.set(y, {"title": get(y).title, "description": get(y).description, "link": get(y).link })
            }
        }
    }

    XmlListModel {
        id: currentStocks
        source: serverDomain+quoteParams
        query: "/stockInfo/currentStats"

        XmlRole { name: "tick"; query: "@stkent0/string()" }
        XmlRole { name: "ask"; query: "@stkent1/string()" }
        XmlRole { name: "buy"; query: "@stkent2/string()" }
        XmlRole { name: "price"; query: "@stkent3/string()" }
        XmlRole { name: "volume"; query: "@stkent4/string()" }
        XmlRole { name: "close"; query: "@stkent5/string()" }
        XmlRole { name: "low_day"; query: "@stkent6/string()" }
        XmlRole { name: "high_day"; query: "@stkent7/string()" }
        XmlRole { name: "open"; query: "@stkent8/string()" }
        XmlRole { name: "low_52_week"; query: "@stkent9/string()" }
        XmlRole { name: "high_52_week"; query: "@stkent10/string()" }
        XmlRole { name: "after_hours"; query: "@stkent11/string()" }
        XmlRole { name: "dividend"; query: "@stkent12/string()" }
        XmlRole { name: "pe_ratio"; query: "@stkent13/string()" }
        XmlRole { name: "eps"; query: "@stkent14/string()" }
        XmlRole { name: "market_cap"; query: "@stkent15/string()" }
        XmlRole { name: "cmpy_name"; query: "@stkent16/string()" }

        property bool firstRunLoaded: false
        onFirstRunLoadedChanged: {
            setChangedFormat()
            sortPage.visible = true  // Turn the entire tab structure to visible true
            if ( parseInt(current_index) > 0 ) { // Only do this if the application started with one or more watched stocks
                chartTick = get(current_index).tick
                currentPage.mainViewCurrent.currentIndex = current_index // Set the index to it as well
            }
            control_tab(sub_tab_index, true) // Load the save subTab index
        }

        onStatusChanged: {  // Rember, the component methods that start with "on" do not evealuate external program signals
            if ( status === XmlListModel.Loading )
                log("Loading currentStocks: "+source)
            if ( status === XmlListModel.Error ) {
                errorText = errorString()+"\n"+i18n.tr("Check your connection and/or firewall settings.")
                connection_error.start()
                log("Error loading "+source+" : "+errorString())
            }
            if ( status === XmlListModel.Ready ) {
                connection_error.stop()
                manageModelList(currentStocksList, currentStocks, true, false)
                firstRunLoaded = true  // run firstLoad before the sort to keep currentIndex correct
                sortModel(currentStocksList)
            }
        }
   }

    XmlListModel {
        id: currentStocksChart
        source: {
            var past = new Date()
            past.setDate(past.getDate()-parseInt(chart_day_range)) // chart_day_range is used as the state for this XMLListView
            serverDomain+"?s="+chartTick+
            "&a="+past.getMonth()+"&b="+past.getDate()+"&c="+past.getFullYear()+
            "&d="+new Date().getMonth()+"&e="+new Date().getDate()+"&f="+new Date().getFullYear()
        }

        query: "/stockInfo/currentStats"
        XmlRole { name: "date"; query: "@stkent0/string()" }
        XmlRole { name: "open"; query: "@stkent1/string()" }
        XmlRole { name: "high"; query: "@stkent2/string()" }
        XmlRole { name: "low"; query: "@stkent3/string()" }
        XmlRole { name: "close"; query: "@stkent4/string()" }
        XmlRole { name: "volume"; query: "@stkent5/string()" }
        onStatusChanged: { // Rember, the component methods that start with "on" do not evealuate external program signals
            if ( status === XmlListModel.Loading )
                log("Loading currentStocksChart: "+source)
            if ( status === XmlListModel.Error ) {
                errorText = errorString()+"\n"+i18n.tr("Check your connection and/or firewall settings.")
                connection_error.start()
                log("Error loading "+source+" : "+errorString())
            }
            if ( status === XmlListModel.Ready ) {
                connection_error.stop()
                chartsPage.repaintCanvas()
            }
        }
    }
    Timer {  // Looks better with a delay (not for functionality). Also used in the canvas component width and height state
         id: onPaintCanvas
         interval: 330; running: false; repeat: false
         onTriggered: chartsPage.repaintCanvas()
    }
    Tabs {
        id: rootTabs
        objectName: "rootTabs"
        anchors.fill: parent
        Tab {
            id: currentstockspage
            anchors.fill: parent
            objectName: "watchedTab"
            title: i18n.tr("Watched")
            page: Page {
                      Item {
                          width: parent.width - 10 // Subtract the main program border amount here
                          height: parent.height
                          anchors.horizontalCenter: parent.horizontalCenter
                          ActivityIndicator {
                              anchors.centerIn: parent
                              clip: false
                              width: units.gu(4)
                              height: width // Keep it square
                              running: !currentStocks.firstRunLoaded
                              onRunningChanged: running && connection_error.start()
                          }
                          Item {
                              id: currentRectSizer
                              objectName: "currentRectSizer"
                              width: parent.width
                              height: Math.round(mainProgram.height/refa)
                              Sort_stocks_page {
                                  id: sortPage
                                  visible: false
                                  Current_stocks_page {
                                      id: currentPage
                                      anchors.top: parent.bottom
                                      anchors.left: parent.left
                                      anchors.right: parent.right
                                      height: currentRectSizer.height - parent.height - subtabsPage.height
                                      Current_subtabs_page {
                                          id: subtabsPage
                                          anchors.top: parent.bottom
                                          anchors.left: parent.left
                                          anchors.right: parent.right
                                          height: units.gu(2)+3
                                      }
                                  }
                              }
                              Current_charts_page {
                                  id: chartsPage
                                  // clip: true
                                  visible: false
                                  anchors.topMargin: units.gu(1)
                                  anchors.bottomMargin: units.gu(2)+2
                                  height: rootTabs.height - parent.height - rootTabs.header.height - anchors.topMargin - anchors.bottomMargin
                                  anchors.top: parent.bottom
                                  anchors.right: parent.right
                                  anchors.left: parent.left
                                  Bottom_splash_page { }
                                  Image {
                                     anchors.horizontalCenter: parent.horizontalCenter
                                     source: Qt.resolvedUrl("svg/poweredby-144x20.svg")
                                     anchors.top: parent.bottom
                                     sourceSize.height: units.gu(2)
                                  }

                              }
                              Current_details_page {
                                  id: detailsPage
                                  clip: true // Don't want to move into other tab
                                  visible: false
                                  anchors.topMargin: units.gu(1)
                                  anchors.bottomMargin: units.gu(1)
                                  height: rootTabs.height - parent.height - rootTabs.header.height - anchors.topMargin - anchors.bottomMargin
                                  anchors.top: parent.bottom
                                  anchors.right: parent.right
                                  anchors.left: parent.left
                                  Bottom_splash_page { }
                              }
                              Current_news_page {
                                  id: newsPage
                                  clip: true // Don't want to move into other tab
                                  visible: false
                                  anchors.topMargin: units.gu(1)
                                  anchors.bottomMargin: units.gu(1)
                                  height: rootTabs.height - parent.height - rootTabs.header.height - anchors.topMargin - anchors.bottomMargin
                                  anchors.top: parent.bottom
                                  anchors.right: parent.right
                                  anchors.left: parent.left
                                  Bottom_splash_page { }
                              }

                          }
                      }
                      tools:
                          ToolbarItems {
                              ToolbarButton {
                                  objectName: "changedDisplay"
                                  action:Action {
                                      text: i18n.tr("Display")
                                      iconSource: changedFormat === '$' ? Qt.resolvedUrl("icons/percent_sign.png") : Qt.resolvedUrl("icons/dollar_sign.png")
                                      onTriggered: setChangedFormat()
                                  }
                              }
                              ToolbarButton {
                                  objectName: "sendTo"
                                  action:Action {
                                      iconSource: Qt.resolvedUrl("icons/external_button.png")
                                      text: i18n.tr("Send to")
                                      onTriggered: {
                                          var cs_tick = ""
                                          for ( var x = 0; x < currentStocksList.count; x++ )
                                              cs_tick += currentStocksList.get(x).tick.toLowerCase()+","
                                          Qt.openUrlExternally("http://finance.yahoo.com/quotes/"+cs_tick)
                                      }
                                 }
                              }
                              ToolbarButton {
                                  objectName: "chartSettings"
                                  action:Action {
                                      iconSource: Qt.resolvedUrl("icons/charts_button.png")
                                      text: i18n.tr("Charts")
                                      onTriggered: { PopupUtils.open(chartSheetConfig) }
                                  }
                              }
                              ToolbarButton {
                                  objectName: "resetData"
                                  action:Action {
                                      iconSource: Qt.resolvedUrl("icons/trash_button.png")
                                      text: i18n.tr("Reset")
                                      onTriggered: { PopupUtils.open(resetApp, mainProgram) }
                                  }
                              }
                          }
                  }
        }
        Tab {
            id: addStockspage
            objectName: "managementTab"
            title: i18n.tr("Management")
            page: Page {
                    Item {
                        width: parent.width - 10 // Subtract the main program border amount here
                        height: parent.height
                        anchors.horizontalCenter: parent.horizontalCenter
                        Add_Stocks_page { id: addStocksTab }
                    }
            }
        }
    }
}
