# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright 2013 Robert Edward Steckroth II <RobertSteckroth@gmail.com> Bust0ut, Surgemcgee

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Robert Steckroth, Surgemcgee Bust0ut <RobertSteckroth@gmail.com>


""" Tests for stock_ticker """

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals, Not, Is
from autopilot.matchers import Eventually
from stock_ticker.tests import StockTickerTestCase


class TestSubtabs(StockTickerTestCase):
    """Tests the Stock Ticker sub tabs functionality"""

    def setUp(self):
        super(TestSubtabs, self).setUp()
        self.assertThat(
            self.ubuntu_sdk.get_qml_view().visible, Eventually(Equals(True)))

    def tearDown(self):
        super(TestSubtabs, self).tearDown() 

       
    def test_chart_subtab(self):
        """Tests if the chart is visible and interacting with the user input."""

        tabs = self.app.select_single("Tabs", objectName="rootTabs")

        self.ubuntu_sdk.switch_to_tab(0)
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(0)))

        self.ubuntu_sdk.after_program_fully_loaded()

        sub_bar = self.app.select_single("QQuickRectangle", objectName='subTabsBar')
        charts_button = sub_bar.select_single('UbuntuShape', objectName='chartsTab')
        details_button = sub_bar.select_single('UbuntuShape', objectName='detailsTab')
    
        subtab_pages = self.app.select_single('QQuickItem', objectName='currentRectSizer').get_children()

        self.ubuntu_sdk.click_object(details_button)
        self.assertThat(subtab_pages[2].visible, Eventually(Equals(True)))

        self.ubuntu_sdk.click_object(charts_button)

        self.assertThat(subtab_pages[2].visible, Eventually(Equals(False)))
        self.assertThat(subtab_pages[1].visible, Eventually(Equals(True)))
      
        chart_dot = self.app.select_single('QQuickImage', objectName='chartDot')
        chart_info = self.app.select_single('QQuickText', objectName='dayInfoText')

        self.ubuntu_sdk.click_object(subtab_pages[1])

        dot_x = chart_dot.x
        info_text = chart_info.text


        self.ubuntu_sdk.move_subtabs_bar_up()

        self.ubuntu_sdk.flick_page_right(subtab_pages[1])

        self.assertThat(chart_dot.x, NotEquals(dot_x))       

        self.assertThat(chart_info.text, NotEquals(info_text))       

        self.assertThat(dot_x < chart_dot.x, Equals(True))       


    def test_details_subtab(self):
        """Tests if the details sub tab is visible and interacting with the user input."""

        tabs = self.app.select_single("Tabs", objectName="rootTabs")

        self.ubuntu_sdk.switch_to_tab(0)
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(0)))

        self.ubuntu_sdk.after_program_fully_loaded()

        sub_bar = self.app.select_single("QQuickRectangle", objectName='subTabsBar')

        news_button = sub_bar.select_single('UbuntuShape', objectName='newsTab')
        details_button = sub_bar.select_single('UbuntuShape', objectName='detailsTab')
    
        subtab_pages = self.app.select_single('QQuickItem', objectName='currentRectSizer').get_children()

        self.ubuntu_sdk.click_object(news_button)
        self.assertThat(subtab_pages[3].visible, Eventually(Equals(True)))

        self.ubuntu_sdk.click_object(details_button)

        self.assertThat(subtab_pages[3].visible, Eventually(Equals(False)))
        self.assertThat(subtab_pages[2].visible, Eventually(Equals(True)))

        details_listview = self.app.select_single('QQuickListView', objectName='detailsListView')

        contentY = details_listview.contentY

        self.assertThat(details_listview.contentY, Eventually(Equals(details_listview.contentY)))

        self.ubuntu_sdk.move_subtabs_bar_up()
        self.ubuntu_sdk.hold_page_down(subtab_pages[2])

        self.assertThat(details_listview.contentY, Eventually(NotEquals(contentY)))      

        self.ubuntu_sdk.release_mouse_press()

        self.assertThat(details_listview.contentY, Eventually(Equals(contentY)))



    def test_news_subtab(self):
        """Tests if the details sub tab is visible and interacting with the user input."""

        tabs = self.app.select_single("Tabs", objectName="rootTabs")

        self.ubuntu_sdk.switch_to_tab(0)
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(0)))

        self.ubuntu_sdk.after_program_fully_loaded()

        sub_bar = self.app.select_single("QQuickRectangle", objectName='subTabsBar')

        news_button = sub_bar.select_single('UbuntuShape', objectName='newsTab')
        details_button = sub_bar.select_single('UbuntuShape', objectName='detailsTab')
    
        subtab_pages = self.app.select_single('QQuickItem', objectName='currentRectSizer').get_children()

        self.ubuntu_sdk.click_object(details_button)
        self.assertThat(subtab_pages[2].visible, Eventually(Equals(True)))

        self.ubuntu_sdk.click_object(news_button)

        self.assertThat(subtab_pages[2].visible, Eventually(Equals(False)))
        self.assertThat(subtab_pages[3].visible, Eventually(Equals(True)))

        news_listview = self.app.select_single('QQuickListView', objectName='newsListView')

        contentY = news_listview.contentY

        self.assertThat(news_listview.contentY, Eventually(Equals(contentY)))

        self.ubuntu_sdk.move_subtabs_bar_up()
        self.ubuntu_sdk.hold_page_down(subtab_pages[3])

        self.assertThat(news_listview.contentY, Eventually(NotEquals(contentY)))      

        self.ubuntu_sdk.release_mouse_press()

        self.assertThat(news_listview.contentY, Eventually(Equals(contentY)))


